import-module .\STColorFunctions.psm1

#Sets up a file dialog prompt for importing files
Function Get-FileName($type)
{
	$username = whoami
	$username = $username.split("\")
	$username = $username[1]
	$initialdirectory = "C:\users\$username\Desktop"
	[System.Reflection.Assembly]::LoadWithPartialName("System.windows.forms")|out-null
	$OpenFileDialog = new-object system.windows.forms.openfiledialog
	$OpenFileDialog.initialdirectory = $initialDirectory
	if($type -eq "csv"){$OpenFileDialog.filter = "CSV (*.csv)|*.csv"}
	if($type -eq "xml"){$OpenFileDialog.filter = "XML (*.xml)|*.xml"}
	$OpenFileDialog.showHelp = $true
	$OpenFileDialog.showDialog()|out-null
	$OpenFileDialog.filename
}

#Can be used to import a CSV of your choosing
Function ImportCSV()
{
	$filepath = Get-FileName csv
	$UserCSV = import-csv $filepath
	return $UserCSV
}

#Can be used to import an XML template of your choosing from the Breaking Point
Function ImportXML()
{
	$filepath = Get-FileName xml
	$UserXML = [xml](Get-content $filepath)
	return $UserXML
}

Function GetInitialDirectory()
{
	$username = whoami
	$username = $username.split("\")
	$username = $username[1]
	$initialdirectory = "C:\users\$username\Desktop"
	return $initialdirectory
}

#Creates the router entries in the XML File.
Function CreateIpsecRouters($CSV,$XML)
{
	#Create the network elements for the ipsec router
	foreach($line in $Csv)
	{
		if ($line.Type -like "IPSecRtr") 
		{
			$newobject = $xml.modelExport.network.networkmodel.element[5].clone()
			
			$Name = $line.name
			$VLAN = $line.container
			$IP = $line.ipaddr
			$Gateway = $line.gateway
			$netmask = $line.count
			
			$newobject.string."#text" = [string]$Name
			$newobject.reference[0]."#text" = [string]"$VLAN"
			$newobject.ip_address[0]."#text" = [string]$gateway
			$newobject.ip_address[1]."#text" = [string]$IP
			$newobject.ip_address[2]."#text" = [string]$IP
			
			$xml.modelExport.network.networkmodel.appendchild($newobject)
		}
	}
	#Create the domain elements based on the hosts requiring IPsec routers (I don't really understand why it gets tied to the host
	
	$x = 6;
	
	Foreach ($i in $csv)
	{
		$MACKEY = "{0:x2}" -f $x
		$CreateMAC = "02:1a:c5:"+$MACKEY+":00:00" 
		
		$VLAN = $i.VLAN
		$NetworkAddr = IPSplit $i.ipaddr
		$HostIP = $i.ipaddr
		$splitIP = $i.ipaddr.split(".")
		$splitPort=$splitIP[1]
		$splitHost=$splitIP[2]
		$RouterIP="10.$splitport.$splithost.2"
		$GatewayIP="10.$splitport.$splithost.1"
		if($splitport -eq "1"){$LCNum = "1";$PortNum="1"}
		if($splitport -eq "2"){$LCNum = "1";$PortNum="2"}
		if($splitport -eq "3"){$LCNum = "2";$PortNum="1"}
		if($splitport -eq "4"){$LCNum = "2";$PortNum="2"}
		$RouterName="LC$LCNum-Port$PortNum-CT$SplitHost-RTR"
		$VLANName="VLAN$Splitport$SplitHost"
		$VLANNumber="$Splitport$SplitHost"
		$Interface="interface $splitport"
		
		If ($i.Type -like "StaticHost" -and $i.container -notlike "VLAN*") 
		{
			$newobject = $xml.modelExport.network.domain[1].clone()
			$newobject.name = [string]$i.Name
			$newobject.endpoints = [string]$i.Name
			$newobject.subnet.router_ip= [string]$RouterIP
			$newobject.subnet.netmask = [string]$i.Count
			$newobject.subnet.netaddr = [string]$networkaddr
			$newobject.subnet.gateway = [string]$GatewayIP
			$newobject.subnet.innervlan = [string]$VLANNumber
			$newobject.subnet.mac_count = "1"
			$newobject.subnet.l2 = $CreateMac
			$newobject.subnet.composition.stack[0].id = [string]$i.Name
			$newobject.subnet.composition.stack[1].id = [string]$i.container
			$newobject.subnet.composition.stack[2].id = [string]$VLANName
			$newobject.subnet.composition.stack[3].id = [string]$Interface
			$newobject.subnet.range.min = [string]$i.ipaddr
			$newobject.subnet.range.max = [string]$i.ipaddr
			
			#Append New Object
			$xml.modelExport.network.appendchild($newobject)
		}
	}
	
	return $xml
}

#Creates the static hosts entries in the XML File with VLAN Containers
Function CreateStaticHostsVlans($CSV, $XML)
{
	####Loop to create IPv4 Elements
	Foreach ($i in $csv)
	{
		If ($i.Type -like "StaticHost") 
		{
		
			$newobject = $xml.modelExport.network.networkModel.element[7].clone()										
			
			$name = $i.name
			$container = $i.container
			$VLAN = $i."VLAN"
			
			$newobject.tags.tag.id = $i.Name
			$newobject.string."#text" = $name
			$newobject.ip_address."#text" = $i.ipaddr
			$newobject.reference."#text" = [string]$container
			
			
			#Append New Object
			$xml.modelExport.network.networkModel.appendchild($newobject)
		
		}
	}
	return $xml
	
}

#Creates the Vlan entries in the XML File
Function CreateVLANs($CSV, $XML)
{	
	$iteration = 1
	$counter = 1
	$x=6
	$UniqueVLANs = $CSV|where{$_.type -like "VLANInt"}

	#Runs through all Unique VLANs and adds them to the XML
	foreach($Line in $UniqueVLANs)
	{
		#Create Variables from Each Line in the Object and format them properly for the XML
		$VLAN = $Line.Name
		$VLANTag = $VLAN.substring(4)
		$Interface = $Line.Container
		$Interface = $Interface.ToLower()
		
		#Convert the VLAN Value to hex and create a properly formed MAC with the key
		$MACKEY = "{0:x2}" -f $x
		$MACKEY2 = "{0:x2}" -f $counter
		$MACKEY3 = "{0:x2}" -f $iteration
		$CreateMAC = "02:1a:c5:00:"+$MACKEY3+":"+$MacKey2 
		
		#Set XML Values for VLANs
		$newobject = $xml.modelExport.network.networkmodel.element[4].clone()
		$newobject.mac_address."#text" = $CreateMac
		$newobject.int[1]."#text" = "$VLANTag"
		$newobject.string."#text" = "$VLAN"	
		$newobject.reference."#text" = $Interface	#Sets the "container" for the VLAN
		
		$xml.modelExport.network.networkmodel.appendchild($newobject)
		$x = $x+1
		if ($counter -ne 200) {$counter = $counter + 1}
		if ($counter -eq 200) {$counter = 1;$iteration = $iteration +1}
	}
	
	#TODO - Currently no good way to enter in Interface Container since it isn't being fed to the ForEach loop. 
	
	return $xml
}

Function IPSPLIT($IPString) #This takes an IP address and turns it into a Network Address
{
		$splits = $ipstring.split(".")
		$Octet1 = $splits[0]
		$Octet2 = $splits[1]
		$Octet3 = $splits[2]
		$Octet4 = $splits[3]
		$NetworkAddress = $splits[0]+"."+$splits[1]+"."+$splits[2]+".0"
		return $NetworkAddress
}



#Makes the window pretty
PrintColorLogo
PrintLine

#Creates an object and a popup window.
$wshell = new-object -ComObject wscript.Shell
$wshell.popup("Please select the CSV to be used",0,"Select a CSV",0x0)

$CSV = ImportCSV
$wshell.popup("Please select the XML to be used",0,"Select a CSV",0x0)
$XML = ImportXML

#Makes changes to the XML Files.
CreateVLANs $CSV $XML
CreateIpsecRouters $CSV $XML
CreateStaticHostsVlans $CSV $XML

#Saves the XML File.
$Desktop = GetInitialDirectory
$FullString = $Desktop + "\test.bpt"
$xml.Save($FullString)
echo 'Your XML file has been saved to your desktop as "test.bpt"'